from .provider import Provider
from .constants import Constants
from ..lib.parser import Parser
from github import Github
import requests
import json
import sys
import os


class GithubProvider(Provider):
    def clone(self, container: str, path: str, options: dict):
        rmFolder = "rm -rf " + self.getContainerName(container)
        command = "git clone --depth=1 https://github.com/" + container + ".git"
        print(rmFolder)
        os.system(rmFolder)

        print(command)
        os.system(command)
        return ""

    # https://pygithub.readthedocs.io/en/latest/examples/Repository.html#get-a-specific-content-file
    def getFile(self, container: str, path: str, options: dict):
        if Constants.OptionsToken in options:
            token = Parser.normalize(options[Constants.OptionsToken])
            # Authorized
            g = Github(token)
        else:
            # Anonymous
            g = Github()

        repo = g.get_repo(container)
        result = repo.get_contents(path)
        response = requests.get(result.raw_data["download_url"])
        return response.json()

    def getContainerName(self, container: str):
        return container.split("/")[-1]



from nimport.providers.constants import Constants


class Provider(object):
    def getFile(self, container: str, path: str, options: dict):
        pass

    def clone(self, container: str, path: str, options: dict):
        pass

    def get(self, container: str, path: str, options: dict):
        if Constants.OptionsClone in options:
            return self.clone(container, path, options)
        else:
            return self.getFile(container, path, options)

    def getContainerName(self, container: str):
        return container



from IPython.display import display, Javascript
from urllib import parse
import nbformat
import json
import os
import subprocess
import shutil
import sys

def open_nb(path, params=None, redirect=True):
    '''
    edit papermill parameters and redirect to new notebook in same project
    '''
    assert os.path.isfile(path)
    assert path.endswith('.ipynb')
    if params:
        # find parameter cell in notebook
        nb = nbformat.read(path, nbformat.NO_CONVERT)
        parameter_cells = [c for c in nb.cells if 'parameters' in c.get(
            'metadata', {}).get('tags', [])]
        pc = parameter_cells[0] if parameter_cells else None
        if pc:
            # overwrite cell contents, write notebook
            pc['source'] = dump_params(params)
            nbformat.write(nb, path)

    # redirect
    if redirect:
        # this sends a blob of javascript to be executed in the nteract window
        js = '''var o = window.location.href.split("/"); o[o.length - 1] = "''' + \
            path + '''"; window.location = o.join("/");'''
        display(Javascript(js))

def clone_repo(repo):
    '''
    clone a git repo, cleaning previous install
    '''
    if not repo:
        return
    assert repo.endswith('.git')
    import shutil
    dirname = repo.split('/')[-1].split('.')[0]
    if os.path.isdir(dirname):
        shutil.rmtree(dirname)
    cmd = 'git clone --depth=1 ' + repo
    subprocess.call(cmd, shell=True)
    assert os.path.isdir(dirname)
    if dirname not in sys.path:
        sys.path.append(dirname)
    return os.path.abspath(dirname)

def parse_params(url):
    '''
    azure notebooks does not allow access to URL parameters.
    this is a cheap function to parse them when provided manually.
    '''
    rc = {}
    if url:
        components = parse.urlsplit(url)
        rc = dict(parse.parse_qsl(components.query))
        rc['url'] = components.scheme + "://" + components.netloc + components.path
    return rc

def dump_params(params):
    lines = [k + ' = ' + json.dumps(v) for k, v in params.items()]
    return '\n'.join(lines)



import json
from urllib.parse import urlparse, ParseResult, urlunparse
from IPython.display import display

ExtensionName = '.ipynb'


def write(content: str, name: str):
    notebookname = name
    display("Creating/updating file.." + notebookname)
    with open(notebookname, 'w') as outfile:
        json.dump(content, outfile)


class NoteBookUrl(object):
    _url: str
    _urlParsed: ParseResult
    _name: str

    @classmethod
    def __init__(self, url: str):
        self._url = url
        self.parse()

    @classmethod
    def parse(self):
        self._urlParsed = urlparse(self._url)
        self._name = self._urlParsed.path.split('/')[-1].split('.')[0]

    @classmethod
    def getName(self):
        return self._name

    @classmethod
    def getNewLink(self, newName: str):
        existing = self._urlParsed
        existingPath = existing.path.split('/')[:-1]
        existingPath.append(newName)
        newPathString = "/".join(existingPath)
        newUrl = ParseResult(scheme=existing.scheme, netloc=existing.netloc,
                             path=newPathString, params='', query='', fragment='')
        return urlunparse(newUrl)
